/*
 * File: rho_key.h
 * Date: 2011-11-01
 *
 * Evsiz (evsiz@this-voice.org)
 * Rho released under the Creative Commons NonCommercial ShareAlike License 3.0
 * See http://creativecommons.org for details
 *
 * Rho is a fork of the Linux port of MicroEMACS developed by Humberto
 * Ortiz-Zuazaga and Jean-Pierre Demailly. Their work forked the MicroEMACS
 * of Daniel M. Lawrence which was in turn a fork of uemacs by Dave G.
 * Conroy. MicroEmacs was copyrighted 1993 by Mr. Lawrence with a
 * stipulation that it could not be used commercially. Uemacs was released
 * by Mr. Conroy into the public domain in 1985.
 *
 */


/***

Table of Key Bindings. It is searched sequentially for
keystrokes. So the more frequent key-strokes are at the top.

NOTE: Altered ^G and ^U bindings must be mirrored in rho_init.h

NOTE: Leaving ^C open allows its use in killing child processes.

***/

KEYTAB keytab[NBINDS] = {


  {CTRL | 'A', BINDFNC, beginning_of_line, NULL},
  {CTRL | 'B', BINDFNC, previous_character, NULL},
/*  {CTRL | 'C', BINDFNC, UNUSED, NULL}, */  /* see NOTE above */
  {CTRL | 'D', BINDFNC, delete_forward, NULL},
  {CTRL | 'E', BINDFNC, end_of_line, NULL},
  {CTRL | 'F', BINDFNC, next_character, NULL},
  {CTRL | 'G', BINDFNC, abort_command, NULL},  /* see NOTE above */
  {CTRL | 'H', BINDFNC, delete_backward, NULL},
  {CTRL | 'I', BINDFNC, tab, NULL},
/*   {CTRL | 'J', BINDFNC, UNUSED, NULL}, */
  {CTRL | 'K', BINDFNC, killtext, NULL},
  {CTRL | 'L', BINDFNC, refresh_scrn, NULL},
  {CTRL | 'M', BINDFNC, newline, NULL},
  {CTRL | 'N', BINDFNC, next_line, NULL},
  {CTRL | 'O', BINDFNC, open_line, NULL},
  {CTRL | 'P', BINDFNC, previous_line, NULL},
  {CTRL | 'Q', BINDFNC, quote, NULL},
  {CTRL | 'R', BINDFNC, risearch, NULL},
  {CTRL | 'S', BINDFNC, fisearch, NULL},
  {CTRL | 'T', BINDFNC, twiddle, NULL},
  {CTRL | 'U', BINDFNC, universal_argument, NULL},  /* see NOTE above */
  {CTRL | 'V', BINDFNC, page_down, NULL},
  {CTRL | 'W', BINDFNC, kill_region, NULL},
  {CTRL | 'X', BINDFNC, ctlx, NULL},
  {CTRL | 'Y', BINDFNC, yank, NULL},
/*   {CTRL | 'Z', BINDFNC, UNUSED, NULL}, */
  {CTRL | '[', BINDFNC, meta_prefix, NULL},
/*  {CTRL | '\\', BINDFNC, UNUSED, NULL},  */
/*  {CTRL | '^', BINDFNC, UNUSED, NULL}, */
  {CTRL | '?', BINDFNC, delete_backward, NULL},


  
/*  {CTLX | ' ', BINDFNC, UNUSED, NULL},  */
  {CTLX | '!', BINDFNC, shell_command, NULL},
  {CTLX | '#', BINDFNC, filter_buffer, NULL},
  {CTLX | '$', BINDFNC, set_fill_column, NULL},
  {CTLX | '(', BINDFNC, begin_macro, NULL},
  {CTLX | ')', BINDFNC, end_macro, NULL},
  {CTLX | '<', BINDFNC, search_regex, NULL},
  {CTLX | '>', BINDFNC, search_again, NULL},
  {CTLX | '=', BINDFNC, cursor_position, NULL},
  {CTLX | '?', BINDFNC, describe_key, NULL},
  {CTLX | '@', BINDFNC, pipe_command, NULL},
  {CTLX | 'A', BINDFNC, set_envar, NULL},
  {CTLX | 'B', BINDFNC, select_buffer, NULL},  
/*   {CTLX | 'C', BINDFNC, UNUSED, NULL}, macroed as suspend-to-shell */
  {CTLX | 'D', BINDFNC,	wordcount, NULL},
  {CTLX | 'E', BINDFNC, execute_macro, NULL},
  {CTLX | 'F', BINDFNC, matching_fence, NULL},
  {CTLX | 'G', BINDFNC,	goto_mark, NULL},
/*   {CTLX | 'H', BINDFNC, UNUSED, NULL}, macroed as online help */
  {CTLX | 'I', BINDFNC, this_error, NULL},
  {CTLX | 'J', BINDFNC, make, NULL}, 
  {CTLX | 'K', BINDFNC, delete_buffer, NULL},
  {CTLX | 'L', BINDFNC, make_clean, NULL},
  {CTLX | 'M', BINDFNC, setmod, NULL},
  {CTLX | 'N', BINDFNC, narrow_region, NULL},
  {CTLX | 'O', BINDFNC, next_window, NULL},
  {CTLX | 'P', BINDFNC, previous_window, NULL},
/*  {CTLX| 'Q', BINDFNC, UNUSED, NULL},  */
/*  {CTLX| 'R', BINDFNC, UNUSED, NULL},  */
/*  {CTLX| 'S', BINDFNC, UNUSED, NULL},  */
/*  {CTLX| 'T', BINDFNC, UNUSED, NULL},  */
  {CTLX | 'U', BINDFNC, next_error, NULL},
  {CTLX | 'V', BINDFNC, describe_envars, NULL},
  {CTLX | 'W', BINDFNC, resize_window, NULL},
  {CTLX | 'X', BINDFNC, next_buffer, NULL},
  {CTLX | 'Y', BINDFNC, widen_region, NULL},
  {CTLX | 'Z', BINDFNC,	create_script, NULL},
/*  {CTLX| '^', BINDFNC, UNUSED, NULL},  */
  {CTLX | '0', BINDFNC, delete_window, NULL},
  {CTLX | '1', BINDFNC, single_window, NULL},
  {CTLX | '2', BINDFNC, split_window, NULL},

/*
 * 	NEXT SEVEN LEFT UNUSED FOR USER-BINDINGS.
 *
 *  	{CTLX|'3',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'4',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'5',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'6',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'7',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'8',		BINDFNC,	UNUSED,	NULL},
 *  	{CTLX|'9',		BINDFNC,	UNUSED,	NULL},
 */


  {CTLX | CTRL | 'A', BINDFNC, append_file, NULL},
  {CTLX | CTRL | 'B', BINDFNC, list_buffers, NULL},
  {CTLX | CTRL | 'C', BINDFNC, exit_rho, NULL},
  {CTLX | CTRL | 'D', BINDFNC, detab, NULL},
  {CTLX | CTRL | 'E', BINDFNC, entab, NULL},
  {CTLX | CTRL | 'F', BINDFNC, find_file, NULL},
  {CTLX | CTRL | 'G', BINDFNC, file_grep, NULL},
  {CTLX | CTRL | 'H', BINDFNC, describe_bindings, NULL},
  {CTLX | CTRL | 'I', BINDFNC, insert_file, NULL},
/*   {CTLX | CTRL | 'J', BINDFNC, UNUSED, NULL},  */
  {CTLX | CTRL | 'K', BINDFNC, macro_to_key, NULL},
  {CTLX | CTRL | 'L', BINDFNC, lowercase_region, NULL},
  {CTLX | CTRL | 'M', BINDFNC, delmode, NULL},
  {CTLX | CTRL | 'N', BINDFNC, window_down, NULL},
/*  {CTLX | CTRL | 'O', BINDFNC, UNUSED, NULL}, */
  {CTLX | CTRL | 'P', BINDFNC, window_up, NULL},
/*  {CTLX | CTRL | 'Q', BINDFNC, UNUSED, NULL},  */
  {CTLX | CTRL | 'R', BINDFNC, read_file, NULL},
  {CTLX | CTRL | 'S', BINDFNC, save_file, NULL},
  {CTLX | CTRL | 'T', BINDFNC, trim, NULL},
  {CTLX | CTRL | 'U', BINDFNC, uppercase_region, NULL},
  {CTLX | CTRL | 'V', BINDFNC, view_file, NULL},
  {CTLX | CTRL | 'W', BINDFNC, write_file, NULL},
  {CTLX | CTRL | 'X', BINDFNC, swap_mark, NULL},
/*  {CTLX | CTRL | 'Z', BINDFNC, UNUSED, NULL},  */
  {CTLX | CTRL | '\\', BINDFNC, save_file, NULL},


/*
 * The META-CTRL or Ctl-Alt series is for sourcing macro files
 */
/*  {META | CTRL | 'A', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'B', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'C', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'D', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'E', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'F', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'G', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'H', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'I', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'J', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'K', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'L', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'M', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'N', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'O', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'P', BINDFNC, UNUSED, NULL}, */
/*  {META | CTRL | 'Q', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'R', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'S', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'T', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'U', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'V', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'W', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'X', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'Y', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | 'Z', BINDFNC, UNUSED, NULL},  */
/*  {META | CTRL | '\\', BINDFNC,UNUSED, NULL},  */



  {META | ' ', BINDFNC, set_mark, NULL},
  {META | '!', BINDFNC, unbind_key, NULL},
  {META | '%', BINDFNC, query_replace, NULL},
  {META | ')', BINDFNC, indent_region, NULL},
  {META | '(', BINDFNC, unindent_region, NULL},
  {META | ',', BINDFNC, back_from_tag, NULL},
  {META | '.', BINDFNC, tag_word, NULL},
  {META | '>', BINDFNC, end_of_buffer, NULL},
  {META | '<', BINDFNC, beginning_of_buffer, NULL},
/*   {META | '[', BINDFNC, UNUSED, NULL}, */
/*   {META | '~', BINDFNC, UNUSED, NULL}, */
  {META | 'A', BINDFNC, apropos, NULL},
  {META | 'B', BINDFNC, previous_word, NULL},
  {META | 'C', BINDFNC, capitalize_word, NULL},
  {META | 'D', BINDFNC, delete_next_word, NULL},
  {META | 'E', BINDFNC, end_of_word, NULL},
  {META | 'F', BINDFNC, next_word, NULL},
  {META | 'G', BINDFNC, goto_line, NULL},
  {META | 'H', BINDFNC, delete_previous_word, NULL},
/*  {META | 'I', BINDFNC,       UNUSED,	NULL}, */
  {META | 'J', BINDFNC,	scroll_next_down, NULL}, 
  {META | 'K', BINDFNC,	scroll_next_up, NULL}, 
  {META | 'L', BINDFNC, lowercase_word, NULL},
  {META | 'M', BINDFNC, setgmode, NULL},
  {META | 'N', BINDFNC, next_paragraph, NULL},
/*  	{META|'O',		BINDFNC,	UNUSED,	NULL},  */
  {META | 'P', BINDFNC, previous_paragraph, NULL},
/*   {META | 'Q', BINDFNC, UNUSED, NULL}, */
  {META | 'R', BINDFNC, unmark_buffer, NULL},
/*   {META | 'S', BINDFNC, UNUSED, NULL}, */
  {META | 'T', BINDFNC, execute_buffer, NULL},
  {META | 'U', BINDFNC, uppercase_word, NULL},
  {META | 'V', BINDFNC, page_up, NULL},
  {META | 'W', BINDFNC, copy_region, NULL},
  {META | 'X', BINDFNC, execute_command, NULL},
  {META | 'Y', BINDFNC,	source, NULL}, 
  {META | 'Z', BINDFNC,	execute_script,	NULL}, 

/*
 * NEXT TEN LEFT UNUSED FOR USER-BINDINGS.
 * (Must be set to nop here, not commented out.)
 */
  {META | '0', BINDFNC, nop, NULL},
  {META | '1', BINDFNC, nop, NULL},
  {META | '2', BINDFNC, nop, NULL},
  {META | '3', BINDFNC, nop, NULL},
  {META | '4', BINDFNC, nop, NULL},
  {META | '5', BINDFNC, nop, NULL},
  {META | '6', BINDFNC, nop, NULL},
  {META | '7', BINDFNC, nop, NULL},
  {META | '8', BINDFNC, nop, NULL}, 
  {META | '9', BINDFNC, nop, NULL},

/* KeyPad */
  {SPEC | '<', BINDFNC, beginning_of_buffer, NULL},	/*  Home  */
  {SPEC | '>', BINDFNC, end_of_buffer, NULL},	        /*  End   */
  {SPEC | 'B', BINDFNC, previous_character, NULL},	/*  Left  */
  {SPEC | 'F', BINDFNC, next_character, NULL},	        /*  Right */
  {SPEC | 'N', BINDFNC, next_line, NULL},	        /*  Down  */
  {SPEC | 'P', BINDFNC, previous_line, NULL},	        /*  Up    */
  {SPEC | 'C', BINDFNC, toggle_overwrite, NULL},	/*  Ins   */
  {SPEC | 'D', BINDFNC, delete_forward, NULL},	        /*  Del   */
  {SPEC | 'V', BINDFNC, page_down, NULL},	        /*  PgDn  */
  {SPEC | 'Z', BINDFNC, page_up, NULL},	                /*  PgUp  */


/* Functions Keys: F1 - F12 */
/*   {SPEC|'1',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'2',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'3',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'4',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'5',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'6',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'7',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'8',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'9',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|'0',	 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'1', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'2', 	BINDFNC,	UNUSED,	NULL},   */


/* Shifted Function Keys F3 - F10  - NOT WORKING AT THIS TIME */
/*   {SPEC|SHFT|'3', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'4', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'5', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'6', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'7', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'8', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'9', 	BINDFNC,	UNUSED,	NULL},   */
/*   {SPEC|SHFT|'0', 	BINDFNC,	UNUSED,	NULL},   */

/* NULL BINDING */
  {0, BINDNUL, NULL, NULL}
};
